﻿#include "StdAfx.h"
#include <Winsock2.h>
#include "COM/Loose_COM.h"
#include "../inc/IBase.h"
#include "MonAx.h"


// --------------------------------------------------------
/// @brief		constructor
// --------------------------------------------------------
MonAx::MonAx(void)
	: h_module_(NULL)
	, p_base_(NULL)
{
}


// --------------------------------------------------------
/// @param[in]	cpIpLocal		IP address of PC (client)
/// @return		cpIpRemote		IP address of Robot controller (server)
///			0	success
/// @brief		initialize
// --------------------------------------------------------
int MonAx::Init(const char* cpIpLocal, const char* cpIpRemote)
{
	bstrIpLocal = cpIpLocal;
	bstrIpRemote = cpIpRemote;

	if(InitSock() < 0) return -1;
	if(InitProxyBase()) return -2;

	return 0;
}


// --------------------------------------------------------
/// @return		0
/// @brief		end
// --------------------------------------------------------
int MonAx::End()
{
	EndProxyBase();
	EndSock();

	return 0;
}


#define SVC_STATUS_BASE				0x4000000
#define SVC_GET_MON_AXIS_ALL		(SVC_STATUS_BASE + 0x10000)	// axis monitoring
#define TOTAL_AXIS					16

// --------------------------------------------------------
/// @return
///			- 0		success
///			- -1	failed
/// @brief		do monitoring
// --------------------------------------------------------
int MonAx::DoMonitoring()
{
	HRESULT		hr;
	int			i, iErrCode;
	int			iOptOut;
	double		daAxisAngle[TOTAL_AXIS];
	double		daCartCoord[6];

	
	// option
	iOptOut = 0x04;		// Joint Angles
	iOptOut |= 0x08;	// TCP(Tool Center Point)
	
	p_base_->InitParam();
	p_base_->PutData(iOptOut);	// input parameter
	
	// service-call
	hr = p_base_->DoProcSync(SVC_GET_MON_AXIS_ALL, &iErrCode);
	if(FAILED(hr)) {
		return -1;
	}

	// joint values (mm or deg.)
	for(i=0; i<TOTAL_AXIS; i++) {
		p_base_->GetData(daAxisAngle[i]);
	}
	printf("J1~J6 (%.2f, %.2f, %.2f, %.2f, %.2f, %.2f) \r\n"
		, daAxisAngle[0], daAxisAngle[1], daAxisAngle[2]
		, daAxisAngle[3], daAxisAngle[4], daAxisAngle[5]);

	// TCP values (X, Y, Z: mm, RX, RY, RZ: deg.)
	for(i=0; i<6; i++) {
		p_base_->GetData(daCartCoord[i]);
	}
	printf("TCP(X=%.2f, Y=%.2f, Z=%.2f, RX=%.2f, RY=%.2f, RZ=%.2f) \r\n"
		, daCartCoord[0], daCartCoord[1], daCartCoord[2]
		, daCartCoord[3], daCartCoord[4], daCartCoord[5]);


	return 0;
}


// --------------------------------------------------------
/// @return
///			0	success
///			-1	failed
/// @brief		startup WINSOCK
// --------------------------------------------------------
int MonAx::InitSock()
{
	printf("InitSock() \r\n");

	// WINSOCK initialization
	WORD wVersionRequested = MAKEWORD(2, 2);	// request up to WinSock version v2.2
	WSADATA wsaData;
	int iret = WSAStartup(wVersionRequested, &wsaData);
	if(iret != 0)
	{
		printf("WSAStartup() failed.\r\n");
		return -1;
	}

	return 0;
}


// --------------------------------------------------------
/// @return		0
/// @brief		cleanup WINSOCK
// --------------------------------------------------------
int MonAx::EndSock()
{
	printf("EndSock() \r\n");

	// Deregister itself from a Windows Sockets implementation 
	WSACleanup();

	return 0;
}



// --------------------------------------------------------
/// @return
///			- 0		success
///			- -1	already loaded
///			- -2	failed in LoadLibrary()
///			- -3	failed in getting interface
/// @brief		Initialize HRRpcProxy library - Base
// --------------------------------------------------------
int MonAx::InitProxyBase()
{
	HRESULT		hr;
	int			iErrCode;
	

	// already loaded
	if(h_module_) return -1;

	// ------------------------------------------
	// DLL dynamic linking
	const char* cpFNameDLL 
#ifdef _WIN64
		= "HRRpcProxy_x64.dll";
#else // 32-bit platform
		= "HRRpcProxy.dll";
#endif
	if((h_module_ = LoadLibraryA(cpFNameDLL)) == NULL)
	{
		printf("Error in loading library\r\n%s", cpFNameDLL);
		return -2;
	}

	// ------------------------------------------
	// create COM component instance & get interface

	// IBase
	hr = CreateInstanceFromInst(h_module_, hr_rpc_proxy::CLSID_Base, hr_rpc_proxy::IID_IBase, (void**)&p_base_);
	if(FAILED(hr)) {
		printf("Error in getting IBase.\r\n");
		return -3;
	}

	// input license key code
	p_base_->SetLicenseKeyCode(12606732, 20180531);

	// open socket
	p_base_->Open(bstrIpLocal, bstrIpRemote, &iErrCode);

	
	return 0;
}


// --------------------------------------------------------
/// @return
///			- 0		success
///			- -1	already unloaded
/// @brief		unload HRRpcProxy DLL library
// --------------------------------------------------------
int MonAx::EndProxyBase()
{
	// already unloaded
	if(h_module_==NULL) return -1;

	// release COM interfaces
	if(p_base_)
	{
		p_base_->Close();	// close socket

		p_base_->Release();
		p_base_ = NULL;
	}

	FreeLibrary(h_module_);
	h_module_ = NULL;
	
	return 0;
}
