﻿// filename : IBase.h
// 
/// @author Choi, Won-Hyuk, Hyundai Heavy Industries Co., Ltd.
/// @section CREATEINFO
///  Creator	: Hyundai Heavy Industries Co., Ltd.
/// @date 2017-08-29

#pragma once

#include <Unknwn.h>
#include <Guiddef.h>
#include "IPose.h"


// ITF ERROR CODES
#define _MK_HS(ofs)			(MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_ITF, ofs))
#define _MK_HE(ofs)			(MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, ofs))

#define BASE_S_ALREADY_DISCONNECTED		_MK_HS(0x000101)		///< Already disconnected.

#define BASE_E_PARAM_EXCEEDED_RANGE		_MK_HE(0x000305)		///< The parameter exceeded its valid range.
#define BASE_E_PARAM_INVALID			_MK_HE(0x000306)		///< The parameter value is invalid.

#define BASE_E_LICENSE_NOT_SET			_MK_HE(0x000401)		///< license code is not set
#define BASE_E_LICENSE_INVALID			_MK_HE(0x000402)		///< license code is invalid
#define BASE_E_LICENSE_EXPIRED			_MK_HE(0x000403)		///< license code is expired
#define BASE_E_NO_NIC_IN_SYSTEM			_MK_HE(0x000411)		///< No network interface card is found in the system.
#define BASE_E_WINSOCK					_MK_HE(0x000412)		///< WinSock error

#define BASE_E_COMM_NOT_CREATED			_MK_HE(0x000501)		///< communication proxy is not created.
#define BASE_E_XDR_ALIGNMENT			_MK_HE(0x000502)		///< XDR data alignment error
#define BASE_E_XDR_EXCEED_BUF			_MK_HE(0x000503)		///< XDR data buffer is exceeded.
#define BASE_E_XDR_NO_MORE_DATA			_MK_HE(0x000504)		///< No more data in XDR data buffer.
#define BASE_E_COMM_REQUEST				_MK_HE(0x000511)		///< communication error in requesting
#define BASE_E_TIME_OUT_RESPONSE		_MK_HE(0x000512)		///< time-out in getting response
#define BASE_E_INVALID_SEGMENT			_MK_HE(0x000513)		///< invalid segment
#define BASE_E_INVALID_DATA_SIZE		_MK_HE(0x000514)		///< invalid data size in segment
#define BASE_E_SERVICE					_MK_HE(0x000520)		///< HR-RPC service error.

#define STATE_E_EVT_NOT_INITIALIZED		_MK_HE(0x000601)		///< event-receiver is not initialized
#define STATE_E_TEMP_FILE_NOT_CREATED	_MK_HE(0x000621)		///< temp. file is not created


namespace hr_rpc_proxy {


///////////////////////////////////////////////////////////
// component ID

#ifdef _WIN64
// {A34FFFF8-9EE6-4015-9E6B-C44416041389}
extern "C" static const CLSID CLSID_Base = 
		{ 0xa34ffff8, 0x9ee6, 0x4015, { 0x9e, 0x6b, 0xc4, 0x44, 0x16, 0x4, 0x13, 0x89 } };

#else // 32-bit platform

// {C9868A49-3B86-4173-90F1-42BA07C2B621}
extern "C" static const CLSID CLSID_Base = 
		{ 0xc9868a49, 0x3b86, 0x4173, { 0x90, 0xf1, 0x42, 0xba, 0x7, 0xc2, 0xb6, 0x21 } };


#endif

///////////////////////////////////////////////////////////
// interface ID
// {9B33B5D1-C312-4127-8527-061F6CBA2AE5}
extern "C" static const IID IID_IBase = 
		{ 0x9b33b5d1, 0xc312, 0x4127, { 0x85, 0x27, 0x6, 0x1f, 0x6c, 0xba, 0x2a, 0xe5 } };


///////////////////////////////////////////////////////////
/**
	@interface	IBase
	@author		Choi, Won-Hyuk, Hyundai Heavy Industries Co., Ltd.
	@brief		Interface of HR-RPC proxy - base
*/
interface IBase : IUnknown
{
public:
	/////////////////////////////////////////////
	// module & component information
	virtual STDMETHODIMP GetModuleHandle(HINSTANCE* pInst) = 0;		///< getting DLL module handle
	virtual STDMETHODIMP GetProgID(BSTR& bsID) = 0;					///< getting ProgID
	virtual STDMETHODIMP GetVerIndProgID(BSTR& bsID) = 0;			///< getting Version-independent ProgID

	// --------------------------------------------------------
	/// @param[in]	ulUserKeyCode	license code corresponding to the system code
	/// @param[in]	ulExpiryDate	license expiry date (e.g. 20160518 is May 18th, 2016)\n
	///								Input the issued expiry date from HHI. 
	///								If different value is input, the key code is invalidated.)
	/// @return		S_OK
	/// @brief		Before use this library's function, valid license key code must be set.
	///				For the license code specific to a system, please contact to supplier.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetLicenseKeyCode(const ULONG ulUserKeyCode, const ULONG ulExpiryDate) = 0;

	// --------------------------------------------------------
	/// @param[in]	bsIpLocal		Local IP address	(ex: "192.168.1.172")
	///								If empty, the system's first NIC's IP address is used.
	/// @param[in]	bsIpRemote		Remote IP address
	/// @param[out]	ipErrCode		pointer to get detailed error code of WinSock error.\n
	///								NULL if not needed.
	///			- WSANOTINITIALISED		Either the application has not called WSAStartup, or WSAStartup failed.
	///			- WSAENETDOWN			A socket operation encountered a dead network
	///			- WSAEAFNOSUPPORT		An address incompatible with the requested protocol was used.
	///			- WSAEINPROGRESS		A blocking operation is currently executing.
	///			- WSAEMFILE				Too many open sockets.
	///			- WSAENOBUFS			An operation on a socket could not be performed because the system lacked sufficient buffer space or because a queue was full.
	///			- WSAEPROTONOSUPPORT	The requested protocol has not been configured into the system, or no implementation for it exists.
	///			- WSAEPROTOTYPE			A protocol was specified in the socket function call that does not support the semantics of the socket type requested.
	///			- WSAESOCKTNOSUPPORT	The support for the specified socket type does not exist in this address family.
	/// @return
	///			- S_OK						success
	///			- E_FAIL					unspecified error
	///			- BASE_E_NO_NIC_IN_SYSTEM	No network interface card is found in the system.
	///			- BASE_E_WINSOCK			WinSock error. Refer to *ipErrCode for detailed error code.
	/// @brief		Reinit(close/create) the socket.
	/// @see		IBase::Close
	// --------------------------------------------------------
    virtual STDMETHODIMP Open(const BSTR bsIpLocal, const BSTR bsIpRemote, int* ipErrCode) = 0;

	// --------------------------------------------------------
	/// @return
	///			- S_OK							success
	///			- BASE_S_ALREADY_DISCONNECTED	Already disconnected.
	/// @brief	Close the socket.
	/// @see	IBase::Init
	// --------------------------------------------------------
	virtual STDMETHODIMP Close() = 0;

	// --------------------------------------------------------
	/// @param[out]	bpRes
	///				- true : The communication socket is created.
	///				- false : The communication socket is destroyed.
	/// @return		S_OK
	// --------------------------------------------------------
	virtual STDMETHODIMP IsOpened(bool* bpRes) = 0;

	// ========================================================
	// Putting input parameter
	
	/// @brief		Initialize parameter set. This function must be called before calling of PutData() functions.
	/// @return		S_OK
	/// @see		IBase::PutData
	virtual STDMETHODIMP InitParam() = 0;

	// --------------------------------------------------------
	/// @param[in]	iDt		data to put
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_EXCEED_BUF		XDR buffer is exceeded.
	/// @brief		Put input parameter - int type
	// --------------------------------------------------------
	virtual STDMETHODIMP PutData(int iDt) = 0;

	// --------------------------------------------------------
	/// @param[in]	dDt		data to put
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_EXCEED_BUF		XDR buffer is exceeded.
	/// @brief		Put input parameter - double type
	// --------------------------------------------------------
	virtual STDMETHODIMP PutData(double dDt) = 0;

	// --------------------------------------------------------
	/// @param[in]	ulDt	data to put
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_EXCEED_BUF		XDR buffer is exceeded.
	/// @brief		Put input parameter - ULONG type
	// --------------------------------------------------------
	virtual STDMETHODIMP PutData(ULONG ulDt) = 0;

	// --------------------------------------------------------
	/// @param[in]	bsDt		data to put
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_EXCEED_BUF		XDR buffer is exceeded.
	/// @brief		Put input parameter - BSTR type
	// --------------------------------------------------------
	virtual STDMETHODIMP PutData(const BSTR bsDt)= 0;

	// --------------------------------------------------------
	/// @param[in]	cpDt		data to put
	/// @param[in]	iNBytes		the number of byte of cpDt
	///							if -1, cpDt is treated as zero-terminated string (UTF-8)
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_EXCEED_BUF		XDR buffer is exceeded.
	/// @brief		Put input parameter - byte array type
	// --------------------------------------------------------
	virtual STDMETHODIMP PutData(const char* cpDt, int iNBytes=-1) = 0;

	// --------------------------------------------------------
	/// @param[in]	vpPose		data to put\n
	///							(caller has to create instance of IPose to pass,
	///							and after finish calling service, caller has to do IPose::Release())
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_EXCEED_BUF		XDR buffer is exceeded.
	/// @brief		Put input parameter - IPose type
	// --------------------------------------------------------
	virtual STDMETHODIMP PutData_Pose(const void* vpPose) = 0;

	// ========================================================
	// Getting output parameter

	// --------------------------------------------------------
	/// @param[out]	iDt		variable to get data
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_NO_MORE_DATA	No more data in XDR data buffer.
	/// @brief		Get output parameter - int type
	// --------------------------------------------------------
	virtual STDMETHODIMP GetData(int& iDt) = 0;

	// --------------------------------------------------------
	/// @param[out]	dDt		variable to get data
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_NO_MORE_DATA	No more data in XDR data buffer.
	/// @brief		Get output parameter - double type
	// --------------------------------------------------------
	virtual STDMETHODIMP GetData(double& dDt) = 0;

	// --------------------------------------------------------
	/// @param[out]	ulDt	variable to get data
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_NO_MORE_DATA	No more data in XDR data buffer.
	/// @brief		Get output parameter - ULONG type
	// --------------------------------------------------------
	virtual STDMETHODIMP GetData(ULONG& ulDt) = 0;

	// --------------------------------------------------------
	/// @param[in]	bstr	variable to get data
	/// @return		description
	/// @brief		Get output parameter - BSTR type
	// --------------------------------------------------------
	virtual STDMETHODIMP GetData(BSTR& bstr) = 0;

	// --------------------------------------------------------
	/// @param[out]	cppDt		variable to get allocated pointer of byte array\n
	///							(caller has to free the memory using CoTaskMemFree())
	/// @param[out]	ipNBytes	variable to get the number of byte of *cppDt\n
	///							(In case of string, this includes null-termination.)
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_NO_MORE_DATA	No more data in XDR data buffer.
	/// @brief		Get output parameter - byte array type
	// --------------------------------------------------------
	virtual STDMETHODIMP GetData(char** cppDt, int* ipNBytes=NULL) = 0;

	// --------------------------------------------------------
	/// @param[out]	vpPose		IPose object to get a pose data\n
	///							(caller has to create instance of IPose to pass,
	///							and after finish using it, caller has to do IPose::Release())
	/// @return
	///			- S_OK						success
	///			- BASE_E_XDR_ALIGNMENT		Invalid alignment in XDR buffer.
	///			- BASE_E_XDR_EXCEED_BUF		XDR buffer is exceeded.
	/// @brief		Put input parameter - IPose type
	// --------------------------------------------------------
	virtual STDMETHODIMP GetData_Pose(void* vpPose) = 0;

	// --------------------------------------------------------
	/// @param[in]	iProcNo		procedure code to call
	/// @param[out]	ipErrCode	pointer to get HR-RPC service error code.\n
	///							NULL if not needed.
	/// @return
	///			- S_OK						success
	///			- BASE_E_COMM_NOT_CREATED	communication proxy is not created.
	///			- BASE_E_COMM_REQUEST		communication error in requesting
	///			- BASE_E_TIME_OUT_RESPONSE	time-out in getting response
	///			- BASE_E_INVALID_SEGMENT	invalid segment
	///			- BASE_E_INVALID_DATA_SIZE	invalid data size in segment
	///			- BASE_E_SERVICE			HR-RPC service error. Refer to *ipErrCode.
	/// @brief		Call remote procedure (synchronous function)
	// --------------------------------------------------------
	virtual STDMETHODIMP DoProcSync(int iProcNo, int* ipErrCode=NULL) = 0;

	// --------------------------------------------------------
	/// @param[out]	iErrCode	error code of previous service-call
	/// @return		S_OK
	/// @brief		Get error code of previous service-call, 
	///				right after IBase::DoProcSync() is called.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetSvcErrCode(int& iErrCode) = 0;
};

};	// namespace hr_rpc_proxy
