﻿// filename : IMngFile.h
// 
/// @author Choi, Won-Hyuk, Hyundai Heavy Industries Co., Ltd.
/// @section CREATEINFO
///  Creator	: Hyundai Heavy Industries Co., Ltd.
/// @date 2017-08-29

#pragma once

#include <Unknwn.h>
#include <Guiddef.h>
#include <string>


// ITF ERROR CODES
#define MNG_FILE_E_FAIL_OPEN_LOCAL_FILE		_MK_HE(0x010301)		///< failed to open file


namespace hr_rpc_proxy {


interface	IBase;
interface	IProgress;


///////////////////////////////////////////////////////////
// component ID

#ifdef _WIN64
// {AE9F3B87-C92E-4794-A8CB-29D1FDDB43C3}
extern "C" static const CLSID CLSID_MngFile = 
		{ 0xae9f3b87, 0xc92e, 0x4794, { 0xa8, 0xcb, 0x29, 0xd1, 0xfd, 0xdb, 0x43, 0xc3 } };

#else // 32-bit platform

// {B7A4EAEA-D3C8-4ca6-95C7-86E20AC2CBF5}
extern "C" static const CLSID CLSID_MngFile = 
		{ 0xb7a4eaea, 0xd3c8, 0x4ca6, { 0x95, 0xc7, 0x86, 0xe2, 0xa, 0xc2, 0xcb, 0xf5 } };

#endif

///////////////////////////////////////////////////////////
// interface ID
// {89986072-580C-4465-9C9C-FC20A7C020DE}
extern "C" static const IID IID_IMngFile = 
		{ 0x89986072, 0x580c, 0x4465, { 0x9c, 0x9c, 0xfc, 0x20, 0xa7, 0xc0, 0x20, 0xde } };


///////////////////////////////////////////////////////////
/**
	@interface	IMngFile
	@author		Choi, Won-Hyuk, Hyundai Heavy Industries Co., Ltd.
	@brief		Interface of HR-RPC proxy - file management
*/
interface IMngFile : IUnknown
{
public:
	/////////////////////////////////////////////
	// module & component information
	virtual STDMETHODIMP GetModuleHandle(HINSTANCE* pInst) = 0;		///< getting DLL module handle
	virtual STDMETHODIMP GetProgID(BSTR& bsID) = 0;					///< getting ProgID
	virtual STDMETHODIMP GetVerIndProgID(BSTR& bsID) = 0;			///< getting Version-independent ProgID

	/////////////////////////////////////////////
	// initialization

	// --------------------------------------------------------
	/// @param[in]	pProxyBase	the interface of IBase
	/// @return		S_OK
	/// @brief		For HR-RPC communication, firstly IBase interface has to be set.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetProxyBase(IBase* pProxyBase) = 0;


	// --------------------------------------------------------
	/// @param[in]	bsPath		Path (ex: "/sdc1/Paint/")
	///							Rightmost '/' may be omitted or not.
	///							Don't use '\\' instead of '/' as separator.
	/// @return		ERROR_NOT_SUPPORTED
	/// @brief	(NOT SUPPORTED YET)\n
	///			Set path.\n
	///			This path is transferred as path parameter of file operation by service caller.\n
	///			If this path is null string, the operations are executed on block-file system.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetPath(const BSTR bsPath) = 0;

	// --------------------------------------------------------
	/// @param[out]	bsPath		variable to take the path.
	/// @return		ERROR_NOT_SUPPORTED
	/// @brief	(NOT SUPPORTED YET)\n
	///			Get path which was set by SetPath()
	// --------------------------------------------------------
	virtual STDMETHODIMP GetPath(BSTR& bsPath) = 0;


	// ========================================================
	// File list

	// --------------------------------------------------------
	/// @return		same as IBase::DoProcSync()
	/// @brief		Update all fileinfo from remote and store them in this object.
	///				To get a stored fileinfo, use GetFileInfo() function.
	/// @see		IMngFile::GetNFileInfo, IMngFile::GetFileInfo
	// --------------------------------------------------------
	virtual STDMETHODIMP UpdateFileInfo() = 0;

	/// @return		Get the number of stored fileinfo.
	// --------------------------------------------------------
	/// @param[out]	iNFile	variable to get the number of file.
	/// @return		same as IBase::DoProcSync()
	/// @brief		After UpdateFileInfo(), get the number of file in file-list.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetNFileInfo(int& iNFile) = 0;

	// --------------------------------------------------------
	/// @param[in]	iIdx		The index of event to get.
	/// @param[out]	bsFName		filename (title.ext)
	/// @param[out]	iSize		file size(the number of block)
	/// @param[out]	bsDate		date ("YYYY/MM/DD" format)
	/// @param[out]	bsTime		time ("HH/MM/SS" format)
	/// @return
	///			- S_OK							Success.
	///			- BASE_E_PARAM_EXCEEDED_RANGE	lIdx is out of bound
	/// @brief	Get lIdx'th fileinfo into pFileInfo.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetFileInfo(int iIdx, BSTR& bsFName, int& iSize, BSTR& bsDate, BSTR& bsTime) = 0;

	// ========================================================
	// Copying / Deleting / Renaming file

	// --------------------------------------------------------
	/// @param[in]	pProgress	An instance of your class inherited from IProgress.
	/// @return		S_OK
	/// @brief		Set an IProgress object to get the progress information
	/// @see		IProgress
	// --------------------------------------------------------
	virtual STDMETHODIMP SetProgress(hr_rpc_proxy::IProgress* pProgress) = 0;

	// --------------------------------------------------------
	/// @param[in]	bsFNameRmt		The source filename of remote.
	/// @param[in]	bsPathNameLoc	The target path/filename of local.
	/// @return
	///			- S_OK								Success.
	///			- MNG_FILE_E_FAIL_OPEN_LOCAL_FILE	Failed to open local file.
	///			- others							same as IBase::DoProcSync()
	/// @brief	Copy the remote robot controller file named strFNameRmt
	///			to local file named strPathNameLoc.
	// --------------------------------------------------------
	virtual STDMETHODIMP ReadFile(const BSTR bsFNameRmt, const BSTR bsPathNameLoc) = 0;
	
	// --------------------------------------------------------
	/// @param[in]	bsFNameRmt		The target filename of remote
	/// @param[in]	bsPathNameLoc	The source path/filename of local.
	/// @return
	///			- S_OK								Success.
	///			- MNG_FILE_E_FAIL_OPEN_LOCAL_FILE	Failed to open local file.
	///			- others							same as IBase::DoProcSync()
	/// @brief	Copy the local file named strPathNameLoc
	///			to remote robot controller file named strFNameRmt. 
	// --------------------------------------------------------
	virtual STDMETHODIMP WriteFile(const BSTR bsFNameRmt, const BSTR bsPathNameLoc) = 0;

	// --------------------------------------------------------
	/// @param[in] bsFNameRmt	The name of remote file to delete.
	/// @return
	/// @return
	///			- S_OK							Success.
	///			- others						same as IBase::DoProcSync()
	/// @brief	Delete the remote robot controller file named strFNameRmt.
	// --------------------------------------------------------
	virtual STDMETHODIMP DeleteFile(const BSTR bsFNameRmt) = 0;

	// --------------------------------------------------------
	/// @param[in] bsFNameOld		The name of remote file to rename
	/// @param[in] bsFNameNew		New file name
	/// @return
	///			- S_OK							Success.
	///			- others						same as IBase::DoProcSync()
	/// @brief	Rename the remote file from strFNameOld to strFNameNew.
	// --------------------------------------------------------
	virtual STDMETHODIMP RenameFile(const BSTR bsFNameOld, const BSTR bsFNameNew) = 0;

	// --------------------------------------------------------
	/// @param[out]	bsErrSubMsg		variable to get the error sub-message
	/// @return		S_OK
	/// @brief		After failure of IMngFile::WriteFile(), get the detailed error message.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetErrSubMsg(BSTR& bsErrSubMsg) = 0;

	/// @return		Get the string failed to be written.
	// --------------------------------------------------------
	/// @param[out]	bsFailedToBeWritten		variable to get the failed string
	/// @return		S_OK
	/// @brief		After failure of IMngFile::WriteFile(), get the string row which failed to be written.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetStrFailedToBeWritten(BSTR& bsFailedToBeWritten) = 0;
};

};	// namespace hr_rpc_proxy