﻿// filename : IPose.h
// 
/// @author Choi, Won-Hyuk, Hyundai Heavy Industries Co., Ltd.
/// @section CREATEINFO
///  Creator	: Hyundai Heavy Industries Co., Ltd.
/// @date 2017-08-29

#pragma once

#include <Unknwn.h>
#include <Guiddef.h>


namespace hr_rpc_proxy {


///////////////////////////////////////////////////////////
// component ID

#ifdef _WIN64
// {27932382-60B2-4f09-B99E-01527C816288}
extern "C" static const CLSID CLSID_Pose = 
		{ 0x27932382, 0x60b2, 0x4f09, { 0xb9, 0x9e, 0x1, 0x52, 0x7c, 0x81, 0x62, 0x88 } };

#else // 32-bit platform

// {2E2B014D-0DE6-4c0d-9D9F-045FECB106EA}
extern "C" static const CLSID CLSID_Pose = 
		{ 0x2e2b014d, 0xde6, 0x4c0d, { 0x9d, 0x9f, 0x4, 0x5f, 0xec, 0xb1, 0x6, 0xea } };

#endif

///////////////////////////////////////////////////////////
// interface ID
// {2744E56E-0D6D-4b84-9E70-CA1735D2736D}
extern "C" static const IID IID_IPose = 
		{ 0x2744e56e, 0xd6d, 0x4b84, { 0x9e, 0x70, 0xca, 0x17, 0x35, 0xd2, 0x73, 0x6d } };


// the number of elements
#define		kNJointMax		16	///< total maximum number of joint
#define		kNCfgElem		10	///< the number of config. element

// coordinate system for ROBOT_CFG
#define CRD_BASE		0	///< BASE coordinate system
#define CRD_ROBOT		1	///< ROBOT coordinate system
#define CRD_ANGLE		2	///< ANGLE coordinate system
#define CRD_ENCODER		3	///< ENCODER coordinate system
#define CRD_USER		4	///< USER coordinate system


///////////////////////////////////////////////////////////
/**
	@interface	IPose
	@author		Choi, Won-Hyuk, Hyundai Heavy Industries Co., Ltd.
	@brief		Interface of HR-RPC proxy - pose
*/
interface IPose : IUnknown
{
public:
	/////////////////////////////////////////////
	// module & component information
	virtual STDMETHODIMP GetModuleHandle(HINSTANCE* pInst) = 0;		///< getting DLL module handle
	virtual STDMETHODIMP GetProgID(BSTR& bsID) = 0;					///< getting ProgID
	virtual STDMETHODIMP GetVerIndProgID(BSTR& bsID) = 0;			///< getting Version-independent ProgID


	// ========================================================
	// Get/Set data

	// --------------------------------------------------------
	/// @param[out]	dpJ			double[kNJointMax] array to get joint value or TCP.\n
	///							The unit is mm or deg.
	///							- in case of joint : J1 ~ J16
	///							- in case of cartesian  : X, Y, Z, RX, RY, RZ, T1~T10
	/// @return		S_OK
	/// @brief		Get joint or TCP values of this object.\n
	///				(In case of encoder coordinate-system, use IPose::GetJ_Raw() instead of this function.)
	// --------------------------------------------------------
	virtual STDMETHODIMP GetJ(double* dpJ) const = 0;

	// --------------------------------------------------------
	/// @param[out]	ipJ			int[kNJointMax] array to get joint value or TCP.\n
	///							The unit is mm or deg.
	///							(x1000-normalized, except in case of encoder value.
	///							for example, if ipJ[0] is -32070, it means X is -32.070[deg])
	///							- in case of joint : J1 ~ J16
	///							- in case of cartesian  : X, Y, Z, RX, RY, RZ, T1~T10
	/// @return		S_OK
	/// @brief		Get joint or TCP values of this object.\n
	///				Hi5/Hi5a controller stores POSE element as fixed-point real value.
	///				If you want to get this raw value, use this function.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetJ_Raw(int* ipJ) const = 0;
	
	// --------------------------------------------------------
	/// @param[out]	ipCfg		int[kNCfgElem] array to get robot-configuration\n
	///				- [0]: AutoCfg
	///				- [1]: FrontBack
	///				- [2]: AboveBelow
	///				- [3]: FlipNonflip
	///				- [4]: LeftRight
	///				- [5]: R2_180
	///				- [6]: R1_180
	///				- [7]: B_180
	///				- [8]: reserved
	///				- [9]: reserved
	/// @param[out]	iCrd		0=base / 1=robot / 3=encoder / 4=user
	/// @param[out]	iUCrdNum		user coordinate number (1~10)
	/// @param[out]	iUnit		unit number (0~7)
	/// @return		S_OK
	/// @brief		Get robot-configuration, coordinate system, and unit number of this object.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetCfg(int* ipCfg, int& iCrd, int& iUCrdNum, int& iUnit) const = 0;
	
	// --------------------------------------------------------
	/// @param[out]	dpJ			double[kNJointMax] array to set joint value or TCP.\n
	///							The unit is mm or deg.
	///							- in case of joint : J1 ~ J16
	///							- in case of cartesian  : X, Y, Z, RX, RY, RZ, T1~T10
	/// @return		S_OK
	/// @brief		Set joint or TCP values of this object.\n
	///				(In case of encoder coordinate-system, use IPose::GetJ_Raw() instead of this function.)
	// --------------------------------------------------------
	virtual STDMETHODIMP SetJ(const double* dpJ) = 0;

	// --------------------------------------------------------
	/// @param[out]	ipJ			int[kNJointMax] array to set joint value or TCP.\n
	///							The unit is mm or deg.
	///							(x1000-normalized, except in case of encoder value.
	///							for example, if ipJ[0] is -32070, it means X is -32.070[deg])
	///							- in case of joint : J1 ~ J16
	///							- in case of cartesian  : X, Y, Z, RX, RY, RZ, T1~T10
	/// @return		S_OK
	/// @brief		Set joint or TCP values of this object.\n
	///				Hi5/Hi5a controller stores POSE element as fixed-point real value.
	///				If you want to set this raw value, use this function.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetJ_Raw(const int* ipJ) = 0;

	// --------------------------------------------------------
	/// @param[in]	ipCfg		int[kNCfgElem] array to set robot-configuration\n
	///				- [0]: AutoCfg
	///				- [1]: FrontBack
	///				- [2]: AboveBelow
	///				- [3]: FlipNonflip
	///				- [4]: LeftRight
	///				- [5]: R2_180
	///				- [6]: R1_180
	///				- [7]: B_180
	///				- [8]: reserved
	///				- [9]: reserved
	/// @param[in]	iCrd		0=base / 1=robot / 3=encoder / 4=user
	/// @param[in]	iUCrdNum		user coordinate number (1~10)
	/// @param[in]	iUnit		unit number (0~7)
	/// @return		S_OK
	/// @brief		Set robot-configuration, coordinate system, and unit number of this object.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetCfg(const int* ipCfg, int iCrd, int iUCrdNum, int iUnit) = 0;

};

};	// namespace hr_rpc_proxy