﻿/// @file		IState.h
// 
/// @author Choi, Won-Hyuk, Hyundai Heavy Industries Co., Ltd.
/// @section CREATEINFO
///  Creator	: Hyundai Heavy Industries Co., Ltd.
/// @date 2017-08-29

#pragma once

#include <Unknwn.h>
#include <Guiddef.h>
#include <string>


using std::string;


namespace hr_rpc_proxy {


interface IBase;

///////////////////////////////////////////////////////////
// component ID

#ifdef _WIN64
// {6FEBFDC5-C88E-40e1-A44B-78069D2588A4}
extern "C" static const CLSID CLSID_State = 
		{ 0x6febfdc5, 0xc88e, 0x40e1, { 0xa4, 0x4b, 0x78, 0x6, 0x9d, 0x25, 0x88, 0xa4 } };

#else // 32-bit platform

// {33901658-E5B2-4dbf-AB1E-CD8259EB6AE2}
extern "C" static const CLSID CLSID_State = 
		{ 0x33901658, 0xe5b2, 0x4dbf, { 0xab, 0x1e, 0xcd, 0x82, 0x59, 0xeb, 0x6a, 0xe2 } };

#endif

///////////////////////////////////////////////////////////
// interface ID
// {818DAFA7-F90C-4db0-803C-B69109FC152E}
extern "C" static const IID IID_IState = 
		{ 0x818dafa7, 0xf90c, 0x4db0, { 0x80, 0x3c, 0xb6, 0x91, 0x9, 0xfc, 0x15, 0x2e } };


/// @brief	The maximum number of X/Y relay indices for each FN(Fieldbus Network) object
#define MAX_FN_XY_IDX		128

/// @brief	I/O, relay types
enum { CLS_X=0, CLS_Y, CLS_DI, CLS_DO, CLS_R, CLS_K, CLS_SP, CLS_T, CLS_C, CLS_M, CLS_S, CLS_FB_X, CLS_FB_Y, CLS_AO, CLS_AI };


///////////////////////////////////////////////////////////
/**
	@interface	IState
	@author		Choi, Won-Hyuk, Hyundai Heavy Industries Co., Ltd.
	@brief		Interface of HR-RPC proxy - state
*/
interface IState : IUnknown
{
public:
	/////////////////////////////////////////////
	// module & component information
	virtual STDMETHODIMP GetModuleHandle(HINSTANCE* pInst) = 0;		///< getting DLL module handle
	virtual STDMETHODIMP GetProgID(BSTR& bsID) = 0;					///< getting ProgID
	virtual STDMETHODIMP GetVerIndProgID(BSTR& bsID) = 0;			///< getting Version-independent ProgID

	/////////////////////////////////////////////
	// initialization

	// --------------------------------------------------------
	/// @param[in]	pProxyBase	the interface of IBase
	/// @return		S_OK
	/// @brief		For HR-RPC communication, firstly IBase interface has to be set.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetProxyBase(IBase* pProxyBase) = 0;


	// ========================================================
	// Reset, start, stop the robot

	// --------------------------------------------------------
	/// @return		same as IBase::DoProcSync()
	/// @brief	Reset the remote robot.
	///			Program counter goes to {main job}/step0/func0
	// --------------------------------------------------------
	virtual STDMETHODIMP Reset() = 0;

	// --------------------------------------------------------
	/// @param[in]	iOpt
	/// -			0 : Stop
	/// -			1 : Start
	/// @param[in]	iExt
	/// -			0 : Internal
	/// -			1 : External ('Program selection' assigned input signals is applied)
	/// -			2 : External ('Program selection' assigned input signals is ignored)
	/// @return		same as IBase::DoProcSync()
	/// @brief	Playback start or stop the remote robot.
	// --------------------------------------------------------
	virtual STDMETHODIMP PBackStartStop(int iOpt, int iExt) = 0;


	// ========================================================
	// event-receiver

	// --------------------------------------------------------
	/// @param[out] iHEvent		get the event-receiver handle
	/// @return		same as IBase::DoProcSync()
	/// @brief	Subscribe to get the events.
	/// @see	IState::EndEvent
	// --------------------------------------------------------
	virtual STDMETHODIMP InitEvent(int& iHEvent) = 0;

	// --------------------------------------------------------
	/// @param[in] iHEvent
	/// -			0	unsubscribe current handle
	/// -			>0	unsubscribe specified handle forcefully
	/// @return		same as IBase::DoProcSync()
	/// @brief	Unsubscribe to get the events.
	/// @see	IState::InitEvent
	// --------------------------------------------------------
	virtual STDMETHODIMP EndEvent(int iHEvent=0) = 0;

	// --------------------------------------------------------
	/// @return
	/// -		STATE_E_EVT_NOT_INITIALIZED		First, IState::InitEvent() has be called.
	/// -		others							same as IBase::DoProcSync()
	/// @brief	Update all events from remote and store them in this object.
	///			To get a stored event, use IState::GetEvent function.
	/// @see	IState::GetNEvent, IState::GetEvent
	// --------------------------------------------------------
	virtual STDMETHODIMP UpdateEvent() = 0;

	/// @param[out]	iNEvent		the number of stored events
	/// @return		S_OK
	virtual STDMETHODIMP GetNEvent(int& iNEvent) = 0;

	// --------------------------------------------------------
	/// @param[in]	iIdx	The index of event to get.
	/// @param[out]	iType	Event type (0=Error, 1=Warning)
	/// @param[out]	iNum	Event number (Error or warning number)
	/// @param[out]	iAux	Event auxiliary data (e.g. # of troublesome axis)
	/// @return
	///				- S_OK		success
	///				- BASE_E_PARAM_EXCEEDED_RANGE		iIdx is out of the number of event
	/// @brief		Get iIdx'th event data
	// --------------------------------------------------------
	virtual STDMETHODIMP GetEvent(int iIdx, int& iType, int& iNum, int& iAux) = 0;


	// ========================================================
	// Updating various state

	// --------------------------------------------------------
	/// @return		same as IBase::DoProcSync()
	/// @brief	Update the state of the remote robot controller.
	///			The information is stored in the object.
	// --------------------------------------------------------
	virtual STDMETHODIMP UpdateState() = 0;


	// ========================================================
	// Program counter

	/// @param[out]	iProgNo		current job number
	/// @param[out]	iStepNo		current step number
	/// @param[out]	iFuncNo		current function number
	/// @return		S_OK
	/// @remarks	Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetProgCnt(int& iProgNo, int& iStepNo, int& iFuncNo) = 0;

	/// @param[out]	iInPosition		In-position state
	/// -		1  : Currently, robot's position is on program counter
	/// -		0  : Currently, robot's position is NOT on program counter
	/// @return		S_OK
	/// @remarks	Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetInPosition(int& iInPosition) = 0;

	// --------------------------------------------------------
	/// @param[in]	iProgNo	: New program(job) number to set.
	/// @param[in]	iTask	: Task number(0~3) to set the job number.
	/// @param[in]	iExt
	/// -			0 : Prohibited in external mode of the robot controller
	/// -			1 : Permitted only in external mode of the robot controller
	/// @return		same as IBase::DoProcSync()
	/// @brief	Select the program(job) number of the remote robot.
	// --------------------------------------------------------
	virtual STDMETHODIMP SelProg(int iProgNo, int iTask, int iExt) = 0;

	// --------------------------------------------------------
	/// @param[in]	iStepNo		New step number to set.
	/// @param[in]	iFuncNo		New function number to set.
	///							if 0, step is selected.
	/// @param[in]	iTask		Task number(0~3) to set the program counter
	/// @return		same as IBase::DoProcSync()
	/// @brief		Select the step/function number of the remote robot.
	///				(Set current program counter)
	///				(Can be selected, regardless of robot controller's current mode)
	// --------------------------------------------------------
	virtual STDMETHODIMP SelStepFunc(int iStepNo, int iFuncNo, int iTask) = 0;

	// ========================================================
	// Mechanism, coordinate systems and tool#
	
	/// @return		Current mechanism number(1~8)
	/// @remarks	Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetMechNo(int& iMechNo) = 0;

	/// @return		Current coordinate system
	/// -			0 : axis
	/// -			1 : cartesian(or user-crd.sys)
	/// -			2 : tool
	/// @remarks	Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetCrdSys(int& iCrdSys) = 0;

	/// @return		Current user coordinate system number(0~10)
	/// @remarks	Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetUCrdNo(int& iUCrdNo) = 0;
	
	/// @return		Current tool number(0~15)
	/// @remarks	Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetToolNo(int& iToolNo) = 0;

	// --------------------------------------------------------
	/// @param[in]	iMechNo		mechanism number(0~7)
	/// @return		same as IBase::DoProcSync()
	///			- BASE_E_PARAM_INVALID		iMechNo is invalid (e.g. not existent)	
	/// @brief		Set the mechanism number
	// --------------------------------------------------------
	virtual STDMETHODIMP SetMechNo(int iMechNo) = 0;

	// --------------------------------------------------------
	/// @param[in]	iCrdSys
	/// -			0 : axis
	/// -			1 : cartesian(or user-crd.sys)
	/// -			2 : tool
	/// -			-1:	next cord.sys
	/// @return		same as IBase::DoProcSync()
	/// @brief	Set coordinate system for jog
	// --------------------------------------------------------
	virtual STDMETHODIMP SetCrdSys(int iCrdSys) = 0;
	
	// --------------------------------------------------------
	/// @param[in]	iUCrdNo		0~10	user coordinate number
	/// @return		same as IBase::DoProcSync()
	/// @brief	Set the user coordinate number
	// --------------------------------------------------------
	virtual STDMETHODIMP SetUCrdNo(int iUCrdNo) = 0;

	// --------------------------------------------------------
	/// @param[out]	ipProgNo	the pointer of int[10] array to receive program numbers
	///							used to define user coordinate systems
	/// @return		same as IBase::DoProcSync()
	/// @brief	get the program numbers to define user coordinate systems
	// --------------------------------------------------------
	virtual STDMETHODIMP GetProgNoUsedToMakeUCrd(int* ipProgNo) = 0;

	// --------------------------------------------------------
	/// @param[in]	iToolNo		0~15	tool number
	/// @return		same as IBase::DoProcSync()
	/// @brief	Set the tool number
	// --------------------------------------------------------
	virtual STDMETHODIMP SetToolNo(int iToolNo) = 0;


	// ========================================================
	// Jog speed
	
	/// @param[out]		iManSpdLev		Current jog speed level(1~8)
	/// @return			S_OK
	/// @remarks		Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetManSpdLev(int& iManSpdLev) = 0;

	/// @param[out]		iJogInching		Whether jog-inching is ON or OFF(1:ON, 0:OFF)
	/// @return			S_OK
	/// @remarks		Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetJogInching(int& iJogInching) = 0;

	// --------------------------------------------------------
	/// @param[in]	iVal	1~8		manual speed level
	/// @return		same as IBase::DoProcSync()
	/// @brief	Set the manual speed level
	// --------------------------------------------------------
	virtual STDMETHODIMP SetManSpdLev(int iVal) = 0;

	// --------------------------------------------------------
	/// @param[in]	iVal
	/// -			0 : jog-inching ON
	/// -			1 : jog-inching OFF
	/// -			-1:	jog-inching TOGGLE
	/// @return		same as IBase::DoProcSync()
	/// @brief	Set the jog-inching state
	// --------------------------------------------------------
	virtual STDMETHODIMP SetJogInching(int iVal) = 0;


	// ========================================================
	// Joystick Jog

	// --------------------------------------------------------
	/// @param[in]	usJogBits0		jog-bits (J7, J8 is available for Hi5a, not Hi5)
	///			- bit 0		J1-	(least significant bit)
	///			- bit 1		J1+
	///			- bit 2		J2-
	///			- bit 3		J2+
	///			- bit 4		J3-
	///			- bit 5		J3+
	///			- bit 6		J4-
	///			- bit 7		J4+

	///			- bit 8		J5-
	///			- bit 9		J5+
	///			- bit 10	J6-
	///			- bit 11	J6+
	///			- bit 12	J7-
	///			- bit 13	J7+
	///			- bit 14	J8-
	///			- bit 15	J8+	(most significant bit)
	/// @param[in]	usJogBits1		reserved 1
	/// @param[in]	usJogBits2		reserved 2
	/// @param[in]	usJogBits3		reserved 3
	/// @return		same as IBase::DoProcSync()
	/// @brief		Joystick Jog (Remote Jog in Automatic Mode)
	// --------------------------------------------------------
	virtual STDMETHODIMP JoystickJog(USHORT usJogBits0, USHORT usJogBits1, USHORT usJogBits2, USHORT usJogBits3) = 0;


	// ========================================================
	// Playback status
	
	/// @param[out]		bOn		whether it is Motor-ON state or not
	/// @return			S_OK
	virtual STDMETHODIMP IsMotorOn(bool& bOn) = 0;
	
	/// @param[out]		bPlayback	whether it is Playback state or not
	/// @return			S_OK
	virtual STDMETHODIMP IsPlayback(bool& bPlayback) = 0;

	// --------------------------------------------------------
	/// @param[out]		iState		The current job state number.
	/// -	-1:	Stop
	/// -	0 :	Playback
	/// -	1 :	StepGo
	/// -	2 :	StepBack
	/// -	3 :	Cmd.IDP.Exe (Command Independant Execution)
	/// @return			S_OK
	/// @remarks	Beforehand, IState::UpdateState should be called.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetJobState(int& iState) = 0;

	// --------------------------------------------------------
	/// @param[out]		iState		The current job sub-state number.
	/// -	0 :	Waiting gun comp
	/// -	1 :	Monitoring interrupt
	/// -	2 :	Operating interrupt
	/// -	3 :	Output end signal
	/// -	4 :	Waiting time delay
	/// -	5 :	Waiting input signal
	/// -	6 :	Shift data wait
	/// -	7 :	L-speed inching
	/// -	8 :	L-speed retract
	/// -	9 :	Work end detection
	/// -	10:	Robot stop wait
	/// -	11:	Inching spd fixed
	/// -	12:	Move to retry pose
	/// -	13:	Return to start P
	/// -	14:	Return to stop P
	/// -	15:	Move to restart P
	/// -	16:	Wait after CND out
	/// -	17:	Waiting stick check
	/// -	18:	Stick recovery
	/// -	19:	Waiting for WCR
	/// -	20:	UNTIL monitoring
	/// -	21:	UNTIL operation
	/// -	22:	Sensor synch wait
	/// -	23:	Executing WAIT
	/// -	24:	Exec. gun search
	/// -	25:	Sq. servo gun
	/// -	26:	Gas preflow
	/// -	27:	Gas postflow
	/// -	28:	Output initial CND
	/// -	29:	Output end CND
	/// -	30:	Moving search S.P
	/// -	31:	Searching start P
	/// -	32:	Moving to start P
	/// -	33:	Rewelding servo G
	/// -	34:	Servo clamping in progress
	/// -	35:	Prog selection wait
	/// -	36:	Waiting robot coop.
	/// -	37:	Waiting coop. start
	/// -	38:	Waiting connecting
	/// -	39:	Waiting disconnecting
	/// -	40:	Mech. connecting
	/// -	41:	Waiting LVS oper.
	/// -	42:	Welding seam tra..
	/// -	43:	Waiting STN relea..
	/// -	44:	Waiting cube entry
	/// -	45:	Waiting retrial
	/// -	46:	Overlap cooperati..
	/// -	47:	ARCON cooperati..
	/// -	48:	Waiting work piece
	/// -	49:	Gas check
	/// -	50:	H-speed inching
	/// -	51:	H-speed retract
	/// -	52:	Subtask end wait
	/// -	53:	Task synch. wait
	/// -	54:	Safe speed moving
	/// -	55:	Path recovery
	/// -	56:	Squeezing Roller Hemming
	/// @return			S_OK
	/// @remarks	Beforehand, IState::UpdateState should be called.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetSubJobState(int& iState) = 0;


	// ========================================================
	// Arc welding status

	/// @param[out]		ulState			Arc welding status
	/// -	0th bit(LSB) :	Arc On
	/// -	1st bit	:		WCR
	/// -	2nd bit	:		Retry
	/// -	3rd bit	:		Overlap
	/// -	4th bit	:		Dry Run
	/// -	5th bit	:		Arc Start
	/// @return			S_OK
	/// @remarks	Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetArcWeldState(ULONG& ulState) = 0;

	/// @param[out]		ulState2			Arc welding status 2
	/// -	LOWORD :		Arc welding current
	/// -	HIWORD :		Arc welding voltage
	/// @return			S_OK
	/// @remarks	Beforehand, IState::UpdateState should be called.
	virtual STDMETHODIMP GetArcWeldState2(ULONG& ulState2) = 0;


	// ========================================================
	// Operating time

	// --------------------------------------------------------
	/// @param[in]	iOpt
	/// -			0 : operation time
	/// -			1 : operation time since system initialization
	/// @return		same as IBase::DoProcSync()
	/// @brief		Update the operation time of the remote robot controller.
	///				The information is stored in the object.
	// --------------------------------------------------------
	virtual STDMETHODIMP UpdateOpTime(int iOpt) = 0;

	/// @param[out]	iDay		Total operating days
	/// @param[out]	iTime		Total operating time(unit: 10msec)
	/// @return		S_OK
	/// @remarks	Beforehand, IState::UpdateOpTime should be called.
	virtual STDMETHODIMP GetTotalOperTime(int& iDay, int& iTime) = 0;

	/// @param[out]	ulDay		Total operating days
	/// @param[out]	ulTime		Total operating time(unit: sec)
	/// @return		S_OK
	/// @remarks	Beforehand, IState::UpdateOpTime should be called.
	virtual STDMETHODIMP GetTotalMotorOnTime(ULONG& ulDay, ULONG& ulTime) = 0;


	// ========================================================
	// Getting current robot pose

	// --------------------------------------------------------
	/// @param[in]	iCrd		Coordinate system as which you get the pose
	/// 						CRD_BASE, CRD_ROBOT, CRD_ANGLE, CRD_ENCODER, CRD_USER
	/// @param[in]	iUCrdNum	user coordinate system number (0~10)
	///							It is used only when iCrd==CRD_USER.
	///	@param[out]	vpPose		Pointer to a IPose instance to get the value
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the stored current(not command) pose into vpPose
	///				as specified coordinate system(iCrd).\n
	///				In case of CRD_USER, iUCrdNum is used.
	///				If iUCrdNum==0, current user coord.# is used.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetCurPose(int iCrd, int iUCrdNum, void* vpPose) = 0;

	// --------------------------------------------------------
	/// @param[in]	iTask		: The number of task (0~3)
	///	@param[out]	dpMmDeg		: Pointer of the variable to get the target values of all joints
	///							: The number of array elements must be >= kNJointMax.
	///							: The unit is mm, deg.
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the target values of all joints into dpMmDeg.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetTgtJointVal(int iTask, double* dpMmDeg) = 0;


	// ========================================================
	// Get/Setting variable value

	// --------------------------------------------------------
	/// @param[in]	cScope	: 'G':global, 'L':local
	/// @param[in]	iIdx	: variable index(1~600)
	/// @param[out]	spVal	: Pointer of the variable to get the short value
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the value of iIdx'th integer variable(V%[iIdx]) into spVal.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetVarInt(char cScope, int iIdx, short* spVal) = 0;

	// --------------------------------------------------------
	/// @param[in]	cScope	: 'G':global, 'L':local
	/// @param[in]	iIdx	: variable index(1~600)
	/// @param[in]	sVal	: short value to set
	/// @return		same as IBase::DoProcSync()
	/// @brief		Set the sVal value to the iIdx'th integer variable(V%[iIdx]).
	// --------------------------------------------------------
	virtual STDMETHODIMP SetVarInt(char cScope, int iIdx, short sVal) = 0;

	// --------------------------------------------------------
	/// @param[in]	cScope	: 'G':global, 'L':local
	/// @param[in]	iIdx	: variable index(1~600)
	/// @param[out]	fpVal	: Pointer of the variable to get the float value
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the value of iIdx'th real variable(V![iIdx]) into fpVal.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetVarFlt(char cScope, int iIdx, float* fpVal) = 0;

	// --------------------------------------------------------
	/// @param[in]	cScope	: 'G':global, 'L':local
	/// @param[in]	iIdx	: variable index(1~600)
	/// @param[in]	fVal	: float value to set
	/// @return		same as IBase::DoProcSync()
	/// @brief		Set the fVal value to the iIdx'th real variable(V![iIdx]).
	// --------------------------------------------------------
	virtual STDMETHODIMP SetVarFlt(char cScope, int iIdx, float fVal) = 0;

	// --------------------------------------------------------
	/// @param[in]	cScope	: 'G':global, 'L':local
	/// @param[in]	iIdx	: variable index(1~999)
	/// @param[out]	bstrVal : variable to get the string
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the value of iIdx'th string variable(V$[iIdx]) into strpVal.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetVarStr(char cScope, int iIdx, BSTR& bstrVal) = 0;

	// --------------------------------------------------------
	/// @param[in]	cScope	: 'G':global, 'L':local
	/// @param[in]	iIdx	: variable index(1~999)
	/// @param[in]	bstrVal	: string value to set
	/// @return		same as IBase::DoProcSync()
	/// @brief		Set the strVal value to the iIdx'th string variable(V$[iIdx]).
	// --------------------------------------------------------
	virtual STDMETHODIMP SetVarStr(char cScope, int iIdx, const BSTR bstrVal) = 0;

	// --------------------------------------------------------
	/// @param[in]	cScope	: 'G':global, 'L':local
	/// @param[in]	iIdx	: variable index(1~9999)
	/// @param[out]	vpPose : pointer to IPose instance to get the pose value
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the value of iIdx'th pose variable(P[iIdx]) into vpPose
	// --------------------------------------------------------
	virtual STDMETHODIMP GetVarPose(char cScope, int iIdx, void* vpPose) = 0;

	// --------------------------------------------------------
	/// @param[in]	cScope	: 'G':global, 'L':local
	/// @param[in]	iIdx	: variable index(1~9999)
	/// @param[in]	vpPose	: pointer to IPose instance to set
	/// @return		same as IBase::DoProcSync()
	/// @brief		Set the vpPose value to the iIdx'th pose variable(P[iIdx]).
	// --------------------------------------------------------
	virtual STDMETHODIMP SetVarPose(char cScope, int iIdx, void* vpPose) = 0;


	// ========================================================
	// Get/Setting relay value

	// --------------------------------------------------------
	/// @param[in]	iObj	: FB1:1, FB3:3, FB5:5, FN1~FN64:1~64
	/// @param[in]	iClass	: CLS_X, CLS_Y, CLS_DI, CLS_DO, CLS_R, CLS_K, CLS_SP
	///						, CLS_T, CLS_C, CLS_M, CLS_S, CLS_FB_X, CLS_FB_Y
	/// @param[in]	iType	: bit:0, char:1, short:2, long:3
	/// @param[in]	iIdx	: 1-based
	/// @param[out]	ipVal	: Pointer to get the value as binary.
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the specified relay's logic value
	///				and transfer it to lpVal as binary.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetRelayValue(int iObj, int iClass, int iType, int iIdx, int* ipVal) = 0;

	// --------------------------------------------------------
	/// @param[in]	iObj	: FB1:1, FB3:3, FB5:5, FN1~FN64:1~64
	/// @param[in]	iClass	: CLS_X, CLS_Y, CLS_DI, CLS_DO, CLS_R, CLS_K, CLS_SP
	///						, CLS_T, CLS_C, CLS_M, CLS_S, CLS_FB_X, CLS_FB_Y
	/// @param[in]	iType	: bit:0, char:1, short:2, long:3
	/// @param[in]	iIdx	: 1-based
	/// @param[in]	iValue	: Relay value to set.
	/// @return		same as IBase::DoProcSync()
	/// @brief		Set the new logic value to the specified relay.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetRelayValue(int iObj, int iClass, int iType, int iIdx, int iValue) = 0;


	// ========================================================
	// Get/Setting alalog signal value

	// --------------------------------------------------------
	/// @param[in]	bOut	: FALSE: in, TRUE: out
	/// @param[in]	iPortNo	: 0~15 (Currently support 0~3. Others are reserved)
	/// @param[out]	dpVolt	: Pointer to get the voltage value. (-12.0V ~ 12.0V)
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the in/output signal value from lPortNo'th analog port.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetAnalogIo(bool bOut, int iPortNo, double* dpVolt) = 0;

	// --------------------------------------------------------
	/// @param[in]	iPortNo	: 0~15 (Currently support 0~3. Others are reserved)
	/// @param[in]	dVolt	: Voltage value to output. (-12.0V ~ 12.0V)
	/// @return		same as IBase::DoProcSync()
	/// @brief		Set the output signal value to lPortNo'th analog port.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetAnalogOutput(int iPortNo, double dVolt) = 0;


	// ========================================================
	// Get/Setting tool data

	// --------------------------------------------------------
	/// @param[in]	iToolNo		tool number (0~15) to get data
	/// @param[out]	dpElem		Pointer to get the 6 element values (X, Y, Z, RX, RY, RZ).\n
	///							The unit is mm, deg.
	/// @return		same as IBase::DoProcSync()
	/// @brief		Get the iToolNo'th tool's position/direction data to dpElem.
	// --------------------------------------------------------
	virtual STDMETHODIMP GetToolData(int iToolNo, double* dpElem) = 0;

	// --------------------------------------------------------
	/// @param[in]	iToolNo		tool number (0~15) to set data
	/// @param[in]	dpElem		Array of the tool data elements(X, Y, Z, RX, RY, RZ) to set.\n
	///							The unit is mm, deg.
	/// @return		same as IBase::DoProcSync()
	/// @brief		Set the iToolNo'th tool's position/direction data with dpElem.
	// --------------------------------------------------------
	virtual STDMETHODIMP SetToolData(int iToolNo, double* dpElem) = 0;
};

};	// namespace hr_rpc_proxy
